from . import Soil

# Clays
soft_clay = Soil(id="soft_clay", name="Soft Clay")
medium_clay = Soil(id="medium_clay", name="Medium Clay")
stiff_clay = Soil(id="stiff_clay", name="Stiff Clay")

# Peats
peat = Soil(id="peat", name="Peat")

# Sands
loose_sand = Soil(id="loose_sand", name="Loose Sand")
stiff_sand = Soil(id="stiff_sand", name="Stiff Sand")
sand = Soil(id="sand", name="Sand")

# Others
gravel = Soil(id="gravel", name="Gravel")
loam = Soil(id="loam", name="Loam")
muck = Soil(id="muck", name="Muck")
undetermined = Soil(id="undetermined", name="Undetermined")
